from django import forms
from utilities.forms.fields import CommentField

from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from .models import Service, ServiceTypeChoices

from tenancy.models import Tenant
from dcim.models import Device, Interface, Cable
from ipam.models import VRF, Prefix, VLAN, ASN, RouteTarget
from vpn.models import L2VPN, Tunnel
from virtualization.models import VirtualMachine


class NewServiceForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = Service
        fields = ('type', 'service_id', 'tenant')


class ServiceRelatedDevicesForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('devices',)

class ServiceRelatedInterfacesForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('interfaces',)

class ServiceRelatedCablesForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('cables',)


class ServiceRelatedVLANsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('vlans',)

class ServiceRelatedPrefixesForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('prefixes',)

class ServiceRelatedVRFsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('vrfs',)

class ServiceRelatedASNsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('asns',)


class ServiceRelatedRouteTargetsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('route_targets',)

class ServiceRelatedL2VPNsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('l2vpns',)

class ServiceRelatedTunnelsForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('tunnels',)

class ServiceRelatedVirtualMachinesForm(forms.ModelForm):

    class Meta:
        model = Service
        fields = ('virtual_machines',)


class ServiceFilterForm(NetBoxModelFilterSetForm):
    comments = CommentField()
    model = Service
    type = forms.MultipleChoiceField(
        choices=ServiceTypeChoices,
        required=False
    )
    service_id = forms.CharField(
        required=False
    )
    tenant = forms.ModelMultipleChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    devices = forms.ModelMultipleChoiceField(
        queryset=Device.objects.all(),
        required=False
    )
    interfaces = forms.ModelMultipleChoiceField(
        queryset=Interface.objects.all(),
        required=False
    )
    cables = forms.ModelMultipleChoiceField(
        queryset=Cable.objects.all(),
        required=False
    )
    vlans = forms.ModelMultipleChoiceField(
        queryset=VLAN.objects.all(),
        required=False
    )
    prefixes = forms.ModelMultipleChoiceField(
        queryset=Prefix.objects.all(),
        required=False
    )
    vrf = forms.ModelMultipleChoiceField(
        queryset=VRF.objects.all(),
        required=False
    )
    asns = forms.ModelMultipleChoiceField(
        queryset=ASN.objects.all(),
        required=False
    )
    route_targets = forms.ModelMultipleChoiceField(
        queryset=RouteTarget.objects.all(),
        required=False
    )
    l2vpns = forms.ModelMultipleChoiceField(
        queryset=L2VPN.objects.all(),
        required=False
    )
    tunnels = forms.ModelMultipleChoiceField(
        queryset=Tenant.objects.all(),
        required=False
    )
    virtual_machines = forms.ModelMultipleChoiceField(
        queryset=VirtualMachine.objects.all(),
        required=False
    )



# Generated by Django 5.2.6 on 2025-09-27 20:31

import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    operations = [
        migrations.CreateModel(
            name='Service',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('type', models.CharField()),
                ('service_id', models.CharField()),
                ('asns', models.ManyToManyField(to='ipam.asn')),
                ('cables', models.ManyToManyField(to='dcim.cable')),
                ('devices', models.ManyToManyField(to='dcim.device')),
                ('interfaces', models.ManyToManyField(to='dcim.interface')),
                ('l2vpns', models.ManyToManyField(to='vpn.l2vpn')),
                ('prefixes', models.ManyToManyField(to='ipam.prefix')),
                ('route_targets', models.ManyToManyField(to='ipam.routetarget')),
                ('tags', taggit.managers.TaggableManager(through='taggit.TaggedItem', to='taggit.Tag')),
                ('tenant', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='tenancy.tenant')),
                ('tunnels', models.ManyToManyField(to='vpn.tunnel')),
                ('virtual_machines', models.ManyToManyField(to='virtualization.virtualmachine')),
                ('vlans', models.ManyToManyField(to='ipam.vlan')),
                ('vrf', models.ManyToManyField(to='ipam.vrf')),
            ],
            options={
                'abstract': False,
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
    ]



from django.urls import path, include
from utilities.urls import get_model_urls
from .views import (
    ServiceView,
    ServiceListView,
    ServiceAddView,
    ServiceEditView,
    ServiceDeleteView,
    ServiceRelatedDevicesView,
    ServiceRelatedInterfacesView,
    ServiceRelatedCablesView,
    ServiceRelatedVLANsView,
    ServiceRelatedPrefixesView,
    ServiceRelatedVRFsView,
    ServiceRelatedASNsView,
    ServiceRelatedRouteTargetsView,
    ServiceRelatedL2VPNsView,
    ServiceRelatedTunnelsView,
    ServiceRelatedVirtualMachinesView,
)

urlpatterns = [
    path('',
         include(get_model_urls('netbox_services', 'service', detail=False))
         ),
    path('<int:pk>/',
         include(get_model_urls('netbox_services', 'service', detail=True))
         ),
    path('<int:pk>/relate/devices/',
         ServiceRelatedDevicesView.as_view(),
         name='relate_devices',
         ),
    path('<int:pk>/relate/interfaces/',
         ServiceRelatedInterfacesView.as_view(),
         name='relate_interfaces',
         ),
    path('<int:pk>/relate/cables/',
         ServiceRelatedCablesView.as_view(),
         name='relate_cables',
         ),
    path('<int:pk>/relate/vlans/',
         ServiceRelatedVLANsView.as_view(),
         name='relate_vlans',
         ),
    path('<int:pk>/relate/prefixes/',
         ServiceRelatedPrefixesView.as_view(),
         name='relate_prefixes',
         ),
    path('<int:pk>/relate/vrfs/',
         ServiceRelatedVRFsView.as_view(),
         name='relate_vrfs',
         ),
    path('<int:pk>/relate/asns/',
         ServiceRelatedASNsView.as_view(),
         name='relate_asns',
         ),
    path('<int:pk>/relate/route_targets/',
         ServiceRelatedRouteTargetsView.as_view(),
         name='relate_route_targets',
         ),
    path('<int:pk>/relate/l2vpns/',
         ServiceRelatedL2VPNsView.as_view(),
         name='relate_l2vpns',
         ),
    path('<int:pk>/relate/tunnels/',
         ServiceRelatedTunnelsView.as_view(),
         name='relate_tunnels',
         ),
    path('<int:pk>/relate/virtual_machines/',
         ServiceRelatedVirtualMachinesView.as_view(),
         name='relate_virtual_machines',
         ),
]



from netbox.api.routers import NetBoxRouter
from .views import ServiceViewSet
app_name = 'netbox_services'
router = NetBoxRouter()
router.register('services', ServiceViewSet)

urlpatterns = router.urls



import django_tables2 as tables

from netbox.tables import NetBoxTable

from .models import Service


class ServiceListTable(NetBoxTable):
    service_id = tables.Column(
        linkify=True,
        verbose_name='Service ID',

    )

    class Meta(NetBoxTable.Meta):
        model = Service
        fields = ('pk', 'service_id', 'type', 'tenant')
        default_columns = ('pk', 'service_id', 'type', 'tenant')


class ServiceTable(NetBoxTable):
    service_id = tables.Column(
        linkify=True,
        verbose_name='Service ID',

    )

    class Meta(NetBoxTable.Meta):
        model = Service
        fields = (
            'type',
            'service_id',
            'tenant',
            'devices',
            'interfaces',
            'cables',
            'vlans',
            'prefixes',
            'vrf',
            'asns',
            'route_targets',
            'l2vpns',
            'tunnels',
            'virtual_machines',
        )
        default_columns = ('pk', 'service_id', 'type', 'tenant')



from netbox.plugins import PluginMenu
from netbox.plugins import PluginMenuItem, PluginMenuButton

services = PluginMenuItem(
    auth_required=True,
    link='plugins:netbox_services:service_list',
    link_text='Business Services',
    buttons=(
        PluginMenuButton(
            link='plugins:netbox_services:service_add',
            title='Add',
            icon_class='mdi mdi-plus',
        ),
    )
)


menu = PluginMenu(
    label='Business Services',
    groups=(
        ('Services', (services,)),
    ),
    icon_class='mdi mdi-receipt-text-arrow-right'
)



from django.db import models
from django.urls import reverse

from netbox.models import NetBoxModel

from tenancy.models import Tenant
from dcim.models import Device, Interface, Cable
from ipam.models import VRF, Prefix, VLAN, ASN, RouteTarget
from vpn.models import L2VPN, Tunnel
from virtualization.models import VirtualMachine

from utilities.choices import ChoiceSet
from taggit.managers import TaggableManager


class ServiceTypeChoices(ChoiceSet):
    CHOICES = [
        ('l2vpn', 'L2VPN'),
        ('l3vpn', 'L3VPN'),
        ('dia', 'DIA'),
        ('transit', 'IP Transit'),
        ('cdn', 'CDN'),
        ('voice', 'Voice')
    ]


class Service(NetBoxModel):
    type = models.CharField(
        choices=ServiceTypeChoices,
        verbose_name='Service Type',
        null=False,
        blank=False
    )
    service_id = models.CharField(
        verbose_name='Service ID',
        unique=True,
        null=False,
        blank=False
    )
    tenant = models.ForeignKey(
        Tenant,
        on_delete=models.PROTECT,
        verbose_name='Service Tenant',
        null=True,
        blank=True
    )
    devices = models.ManyToManyField(
        Device,
        verbose_name='Related Devices',
    )
    interfaces = models.ManyToManyField(
        Interface,
        verbose_name='Related interfaces',
    )
    cables = models.ManyToManyField(
        Cable,
        verbose_name='Related Cables/XConnects',
    )
    vlans = models.ManyToManyField(
        VLAN,
        verbose_name='Related VLANs',
    )
    prefixes = models.ManyToManyField(
        Prefix,
        verbose_name='Related IP Prefixes',
    )
    vrfs = models.ManyToManyField(
        VRF,
        verbose_name='Related VRF',
        blank=True
    )
    asns = models.ManyToManyField(
        ASN,
        verbose_name='Related Autonomous Systems',
    )
    route_targets = models.ManyToManyField(
        RouteTarget,
        verbose_name='Related Route Targets',
    )
    l2vpns = models.ManyToManyField(
        L2VPN,
        verbose_name='Related Route Targets',
    )
    tunnels = models.ManyToManyField(
        Tunnel,
        verbose_name='Related Tunnels',
    )
    virtual_machines = models.ManyToManyField(
        VirtualMachine,
        verbose_name='Related Virtual Machines',
    )
    tags = TaggableManager(
        related_name='netbox_services_service_set',
    )

    class Meta:
        verbose_name = 'Service'
        verbose_name_plural = 'Services'
        ordering = ['service_id']

    def __str__(self):
        return f"{self.service_id}"

    def get_absolute_url(self):
        return f"/plugins/services/{self.pk}/"



from rest_framework import serializers

from netbox.api.serializers import NetBoxModelSerializer
from netbox_services.models import Service

class ServiceSerializer(NetBoxModelSerializer):

    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_services-api:service-detail'
    )
    class Meta:
        model = Service
        fields = ('type',
                  'service_id',
                  'tenant',
                  'tags',
                  'custom_fields',
                  'created',
                  'last_updated',
                  )



# Generated by Django 5.2.6 on 2025-10-01 19:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dcim', '0215_rackreservation_status'),
        ('ipam', '0082_add_prefix_network_containment_indexes'),
        ('netbox_services', '0002_alter_service_asns_alter_service_cables_and_more'),
        ('virtualization', '0048_populate_mac_addresses'),
        ('vpn', '0009_remove_redundant_indexes'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='service',
            options={'ordering': ['service_id']},
        ),
        migrations.AlterField(
            model_name='service',
            name='asns',
            field=models.ManyToManyField(to='ipam.asn'),
        ),
        migrations.AlterField(
            model_name='service',
            name='cables',
            field=models.ManyToManyField(to='dcim.cable'),
        ),
        migrations.AlterField(
            model_name='service',
            name='devices',
            field=models.ManyToManyField(to='dcim.device'),
        ),
        migrations.AlterField(
            model_name='service',
            name='interfaces',
            field=models.ManyToManyField(to='dcim.interface'),
        ),
        migrations.AlterField(
            model_name='service',
            name='l2vpns',
            field=models.ManyToManyField(to='vpn.l2vpn'),
        ),
        migrations.AlterField(
            model_name='service',
            name='prefixes',
            field=models.ManyToManyField(to='ipam.prefix'),
        ),
        migrations.AlterField(
            model_name='service',
            name='route_targets',
            field=models.ManyToManyField(to='ipam.routetarget'),
        ),
        migrations.AlterField(
            model_name='service',
            name='tunnels',
            field=models.ManyToManyField(to='vpn.tunnel'),
        ),
        migrations.AlterField(
            model_name='service',
            name='virtual_machines',
            field=models.ManyToManyField(to='virtualization.virtualmachine'),
        ),
        migrations.AlterField(
            model_name='service',
            name='vlans',
            field=models.ManyToManyField(to='ipam.vlan'),
        ),
    ]



from importlib.metadata import metadata

from netbox.plugins import PluginConfig


plugin = metadata('netbox_services')


class Services(PluginConfig):
    name = plugin.get('Name').replace('-', '_')
    verbose_name = plugin.get('Name').replace('-', ' ').title()
    description = plugin.get('Summary')
    version = plugin.get('Version')
    author = 'Arturo Baldo'
    author_email = 'baldoarturo@gmail.com'
    base_url = 'services'
    min_version = '4.4.0'

config = Services



from utilities.views import register_model_view
from netbox.views.generic import ObjectView, ObjectDeleteView, ObjectEditView, ObjectListView

from .models import Service
from .tables import ServiceListTable
from .forms import NewServiceForm
from .forms import (
    ServiceRelatedDevicesForm,
    ServiceRelatedInterfacesForm,
    ServiceRelatedCablesForm,
    ServiceRelatedVLANsForm,
    ServiceRelatedPrefixesForm,
    ServiceRelatedVRFsForm,
    ServiceRelatedASNsForm,
    ServiceRelatedRouteTargetsForm,
    ServiceRelatedL2VPNsForm,
    ServiceRelatedTunnelsForm,
    ServiceRelatedVirtualMachinesForm
)

#
#   GENERIC VIEWS
#


@register_model_view(Service, name='view', path='', detail=True)
class ServiceView(ObjectView):
    queryset = Service.objects.all()
    template_name = 'netbox_services/service.html'


@register_model_view(Service, name='list', path='', detail=False)
class ServiceListView(ObjectListView):
    queryset = Service.objects.all()
    table = ServiceListTable
    template_name = 'generic/object_list.html'


@register_model_view(Service, name='add', path='add', detail=False)
class ServiceAddView(ObjectEditView):
    queryset = Service.objects.all()
    form = NewServiceForm


@register_model_view(Service, name='edit', detail=True)
class ServiceEditView(ObjectEditView):
    queryset = Service.objects.all()
    form = NewServiceForm


@register_model_view(Service, name='delete', detail=True)
class ServiceDeleteView(ObjectDeleteView):
    queryset = Service.objects.all()

#
#   RELATION VIEWS
#


class ServiceRelatedDevicesView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedDevicesForm


class ServiceRelatedInterfacesView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedInterfacesForm


class ServiceRelatedCablesView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedCablesForm


class ServiceRelatedVLANsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedVLANsForm


class ServiceRelatedPrefixesView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedPrefixesForm


class ServiceRelatedVRFsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedVRFsForm


class ServiceRelatedASNsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedASNsForm


class ServiceRelatedRouteTargetsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedRouteTargetsForm


class ServiceRelatedL2VPNsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedL2VPNsForm


class ServiceRelatedTunnelsView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedTunnelsForm


class ServiceRelatedVirtualMachinesView(ObjectEditView):
    queryset = Service.objects.all()
    form = ServiceRelatedVirtualMachinesForm



# Generated by Django 5.2.6 on 2025-10-02 19:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_services', '0003_alter_service_options_alter_service_asns_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='service',
            old_name='vrf',
            new_name='vrfs',
        ),
        migrations.AlterField(
            model_name='service',
            name='service_id',
            field=models.CharField(unique=True),
        ),
    ]



from netbox.api.viewsets import NetBoxModelViewSet

from .. import models
from .serializers import ServiceSerializer

class ServiceViewSet(NetBoxModelViewSet):
    queryset = models.Service.objects.prefetch_related('tags')
    serializer_class = ServiceSerializer
